package org.jboss.portletbridge.seam;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.security.Principal;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.context.FacesContext;
import javax.portlet.faces.BridgeUtil;
import javax.security.auth.login.LoginException;

import org.jboss.portletbridge.util.BridgeLogger;
import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Install;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;
import org.jboss.seam.annotations.intercept.BypassInterceptors;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.Events;
import org.jboss.seam.security.Identity;

@SuppressWarnings("serial")
@Name("org.jboss.seam.security.identity")
@Scope(ScopeType.SESSION)
@BypassInterceptors
// @Install(value=false, precedence = Install.FRAMEWORK, classDependencies =
// "javax.portlet.Portlet")
@Startup
public class PortalIdentity extends Identity {

	private static final Logger log = BridgeLogger.SEAM.getLogger();

	private JBossLoginDelegate loginDelegate = new JBossLoginDelegate();


	private boolean isPortletPhase() {
		FacesContext facesContext = FacesContext.getCurrentInstance();
		return null != facesContext ? BridgeUtil.isPortletRequest() : false;
	}

	/**
	 * Attempts to authenticate the user. This method is distinct to the
	 * authenticate() method in that it raises events in response to whether
	 * authentication is successful or not. The following events may be raised
	 * by calling login():
	 * <p/>
	 * org.jboss.seam.security.loginSuccessful - raised when authentication is
	 * successful org.jboss.seam.security.loginFailed - raised when
	 * authentication fails org.jboss.seam.security.alreadyLoggedIn - raised if
	 * the user is already authenticated
	 * 
	 * @return String returns "loggedIn" if user is authenticated, or null if
	 *         not.
	 */
	@Override
	public String login() {
		if (isPortletPhase()) {
			try {
				authenticate();

				if (!isLoggedIn()) {
					throw new LoginException();
				}

				if (log.isLoggable(Level.FINE)) {
					log.fine("Login successful for: " + getUsername());
				}

				if (Events.exists()) {
					Events.instance().raiseEvent(EVENT_LOGIN_SUCCESSFUL);
				}
				return "loggedIn";
			} catch (LoginException ex) {

				if (log.isLoggable(Level.FINE)) {
					log.log(Level.FINE, "Login failed for: " + getUsername(),
					        ex);
				}
				if (Events.exists()) {
					Events.instance().raiseEvent(EVENT_LOGIN_FAILED, ex);
				}
			}

			return null;

		} else {
			return super.login();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jboss.seam.security.Identity#authenticate()
	 */
	@Override
	public void authenticate() throws LoginException {
		if (isPortletPhase() && null == getJaasConfigName()) {
			if (!isLoggedIn()) {
				preAuthenticate();
				loginDelegate.login(getUsername(), getPassword());
				postAuthenticate();
			}
		} else {
			super.authenticate();
		}
	}

	@Override
	public void logout() {
		loginDelegate.logout();
		super.logout();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jboss.seam.security.Identity#hasRole(java.lang.String)
	 */
	@Override
	public boolean hasRole(String role) {
		if (isPortletPhase()) {
			return FacesContext.getCurrentInstance().getExternalContext()
			        .isUserInRole(role);
		} else {
			return super.hasRole(role);
		}
	}

	public String getUsername() {
		String username = super.getUsername();
		if (isPortletPhase()) {
			UserInfo userInfo = (UserInfo) Component
			        .getInstance("org.jboss.portletbridge.userInfo");
			String portalUserName = userInfo.get(UserInfo.USER_NAME_FULL);
         //temporary hack for jboss portal not returning username
         if (portalUserName == null){
             portalUserName = getPrincipal().getName();
         }
			if ((null != portalUserName && !portalUserName.equals(username))
			        || (null == portalUserName && null != username)) {
				setUsername(portalUserName);
				username = portalUserName;
			}
		}
		return username;
	}

	public Principal getPrincipal() {
		if (isPortletPhase()) {
			Principal bridgePrincipal = FacesContext.getCurrentInstance()
			        .getExternalContext().getUserPrincipal();
			if (null != bridgePrincipal) {
				Set<Principal> principals = getSubject().getPrincipals();
				if (!principals.contains(bridgePrincipal)) {
					principals.add(bridgePrincipal);
				}
			}
			return bridgePrincipal;
		} else {
			return super.getPrincipal();
		}
	}

	// =============================================================================
	// Portal engine or other portlets can access to Seam components outside of
	// context
	// ( Liferay portal does it, for example ), where authorization methods are
	// always failed.
	// To prevent such error we bypass authorization in that case. Interceptors
	// can't be used
	// in that case because they are required Seam context too.
	// TODO - design more convininient way.
	// =============================================================================

	@Override
	public void checkPermission(String name, String action, Object... arg) {
		if (isContextActive()) {
			super.checkPermission(name, action, arg);

		}
	}

	@Override
	public void checkRestriction(String expr) {
		if (isContextActive()) {
			super.checkRestriction(expr);

		}
	}

	@Override
	public void checkRole(String role) {
		if (isContextActive()) {
			super.checkRole(role);

		}
	}

	private boolean isContextActive() {
		return Contexts.isApplicationContextActive()
		        && null != FacesContext.getCurrentInstance();
	}

	/**
	 * jboss login delegate
	 * 
	 * @author Egor Kolesnikov
	 */
	private static class JBossLoginDelegate implements Serializable {
		private Class<?> authenticationClass;
		private Object jbossAuthentication;

		public JBossLoginDelegate() {
			try {
				authenticationClass = Class
				        .forName("org.jboss.web.tomcat.security.login.WebAuthentication");
				try {
					jbossAuthentication = authenticationClass.newInstance();
				} catch (Exception e) {
					log.log(Level.SEVERE,
					        "JBoss Web Authentication instantiation "
					                + "exception, Web Authentication disabled",
					        e);
				}
			} catch (ClassNotFoundException ex) {
				log.info("JBoss Web Authentication is not available...");
			}
		}

		public void login(String username, String password) {
			if (jbossAuthentication != null) {
				try {
					Method m = authenticationClass.getMethod("login",
					        String.class, Object.class);
					m.invoke(jbossAuthentication, username, password);
				} catch (Exception ex) {
					log.log(Level.SEVERE,
					        "Error logging out with JBoss Web Authentication",
					        ex);
				}
			}
		}

		public void logout() {
			if (jbossAuthentication != null) {
				try {
					Method m = authenticationClass.getMethod("logout");
					m.invoke(jbossAuthentication);
				} catch (Exception ex) {
					log.log(Level.SEVERE,
					        "Error logging out with JBoss Web Authentication",
					        ex);
				}
			}
		}
	}

}
